@@ -23,6 +23,7 @@ from kuryr_kubernetes import config
 
 _DRIVER_NAMESPACE_BASE = 'kuryr_kubernetes.controller.drivers'
 _DRIVER_MANAGERS = {}
+_MULTI_VIF_DRIVERS = []
 
 
 class DriverBase(object):
@@ -50,20 +51,21 @@ class DriverBase(object):
     """
 
     @classmethod
-    def get_instance(cls):
+    def get_instance(cls, driver_alias=None):
         """Get an implementing driver instance."""
 
         alias = cls.ALIAS
-
+        driver_name = alias + '_driver' if not driver_alias else driver_alias
         try:
-            manager = _DRIVER_MANAGERS[alias]
+            manager = _DRIVER_MANAGERS[driver_name]
         except KeyError:
-            name = config.CONF.kubernetes[alias + '_driver']
+            name = (config.CONF.kubernetes[driver_name] if not driver_alias
+                    else driver_alias)
             manager = stv_driver.DriverManager(
                 namespace="%s.%s" % (_DRIVER_NAMESPACE_BASE, alias),
                 name=name,
                 invoke_on_load=True)
-            _DRIVER_MANAGERS[alias] = manager
+            _DRIVER_MANAGERS[driver_name] = manager
 
         driver = manager.driver
         if not isinstance(driver, cls):
@@ -74,6 +76,9 @@ class DriverBase(object):
                             'type': cls})
         return driver
 
+    def __str__(self):
+        return self.__class__.__name__
+
 
 @six.add_metaclass(abc.ABCMeta)
 class PodProjectDriver(DriverBase):
@@ -109,6 +114,23 @@ class ServiceProjectDriver(DriverBase):
         raise NotImplementedError()
 
 
+@six.add_metaclass(abc.ABCMeta)
+class NamespaceProjectDriver(DriverBase):
+    """Provides an OpenStack project ID for Kubernetes Namespace."""
+
+    ALIAS = 'namespace_project'
+
+    @abc.abstractmethod
+    def get_project(self, namespace):
+        """Get an OpenStack project ID for Kubernetes Namespace.
+
+        :param service: dict containing Kubernetes Namespace object
+        :return: project ID
+        """
+
+        raise NotImplementedError()
+
+
 @six.add_metaclass(abc.ABCMeta)
 class PodSubnetsDriver(DriverBase):
     """Provides subnets for Kubernetes Pods."""
@@ -128,6 +150,33 @@ class PodSubnetsDriver(DriverBase):
         """
         raise NotImplementedError()
 
+    def create_namespace_network(self, namespace, project_id):
+        """Create network resources for a namespace.
+
+        :param namespace: string with the namespace name
+        :param project_id: OpenStack project ID
+        :return: dict with the keys and values for the CRD spec, such as
+                 routerId or subnetId
+        """
+        raise NotImplementedError()
+
+    def delete_namespace_subnet(self, kuryr_net_crd):
+        """Delete network resources associated to a namespace.
+
+        :param kuryr_net_crd: kuryrnet CRD obj dict that contains Neutron's
+                              network resources associated to a namespace
+        """
+        raise NotImplementedError()
+
+    def rollback_network_resources(self, crd_spec, namespace):
+        """Rollback created network resources for a namespace.
+
+        :param crd_spec: dict with the keys and values for the CRD spec, such
+                         as routerId or subnetId
+        :param namespace: name of the Kubernetes namespace object
+        """
+        raise NotImplementedError()
+
 
 @six.add_metaclass(abc.ABCMeta)
 class ServiceSubnetsDriver(DriverBase):
@@ -165,6 +214,24 @@ class PodSecurityGroupsDriver(DriverBase):
         """
         raise NotImplementedError()
 
+    def create_namespace_sg(self, namespace, project_id, crd_spec):
+        """Create security group resources for a namespace.
+
+        :param namespace: string with the namespace name
+        :param project_id: OpenStack project ID
+        :param crd_spec: dict with the keys and values for the CRD spec, such
+                         as subnetId or subnetCIDR
+        :return: dict with the keys and values for the CRD spec, such as sgId
+        """
+        raise NotImplementedError()
+
+    def delete_sg(self, sg_id):
+        """Delete security group associated to a namespace.
+
+        :param sg_id: OpenStack security group ID
+        """
+        raise NotImplementedError()
+
 
 @six.add_metaclass(abc.ABCMeta)
 class ServiceSecurityGroupsDriver(DriverBase):
@@ -298,17 +365,68 @@ class PodVIFDriver(DriverBase):
         raise NotImplementedError()
 
 
+@six.add_metaclass(abc.ABCMeta)
+class MultiVIFDriver(DriverBase):
+    """Manages additional ports of Kubernetes Pods."""
+
+    ALIAS = 'multi_vif'
+
+    @abc.abstractmethod
+    def request_additional_vifs(
+            self, pod, project_id, security_groups):
+        """Links Neutron ports to pod and returns them as a list of VIF objects.
+
+        Implementing drivers must be able to parse the additional interface
+        definition from pod. The format of the definition is up to the
+        implementation of each driver. Then implementing drivers must invoke
+        the VIF drivers to either create new Neutron ports on each request or
+        reuse available ports when possible.
+
+        :param pod: dict containing Kubernetes Pod object
+        :param project_id: OpenStack project ID
+        :param security_groups: list containing security groups' IDs as
+                                returned by
+                                `PodSecurityGroupsDriver.get_security_groups`
+        :return: VIF object list
+        """
+        raise NotImplementedError()
+
+    @classmethod
+    def get_enabled_drivers(cls):
+        if _MULTI_VIF_DRIVERS:
+            pass
+        else:
+            drivers = config.CONF.kubernetes['multi_vif_drivers']
+            for driver in drivers:
+                _MULTI_VIF_DRIVERS.append(cls.get_instance(driver))
+        return _MULTI_VIF_DRIVERS
+
+
 class LBaaSDriver(DriverBase):
-    """Manages Neutron/Octavia load balancer to support Kubernetes Services."""
+    """Base class for Openstack loadbalancer services."""
 
     ALIAS = 'endpoints_lbaas'
 
     @abc.abstractmethod
-    def ensure_loadbalancer(self, endpoints, project_id, subnet_id, ip,
+    def get_service_loadbalancer_name(self, namespace, svc_name):
+        """Generate name of a load balancer that represents K8S service.
+
+        In case a load balancer represents K8S service/ep, the handler
+        should call first this API to get the load balancer name and use the
+        return value of this function as 'name' parameter for the
+        'ensure_loadbalancer' function
+
+        :param namespace: K8S service namespace
+        :param svc_name: K8S service name
+        """
+        raise NotImplementedError()
+
+    @abc.abstractmethod
+    def ensure_loadbalancer(self, name, project_id, subnet_id, ip,
                             security_groups_ids, service_type):
         """Get or create load balancer.
 
-        :param endpoints: dict containing K8s Endpoints object
+        :param name: LoadBlancer name
         :param project_id: OpenStack project ID
         :param subnet_id: Neutron subnet ID to host load balancer
         :param ip: IP of the load balancer
@@ -319,22 +437,20 @@ class LBaaSDriver(DriverBase):
         raise NotImplementedError()
 
     @abc.abstractmethod
-    def release_loadbalancer(self, endpoints, loadbalancer):
+    def release_loadbalancer(self, loadbalancer):
         """Release load balancer.
 
         Should return without errors if load balancer does not exist (e.g.
         already deleted).
 
-        :param endpoints: dict containing K8s Endpoints object
         :param loadbalancer: `LBaaSLoadBalancer` object
         """
         raise NotImplementedError()
 
     @abc.abstractmethod
-    def ensure_listener(self, endpoints, loadbalancer, protocol, port):
+    def ensure_listener(self, loadbalancer, protocol, port):
         """Get or create listener.
 
-        :param endpoints: dict containing K8s Endpoints object
         :param loadbalancer: `LBaaSLoadBalancer` object
         :param protocol: listener's protocol (only TCP is supported for now)
         :param port: listener's port
@@ -342,70 +458,175 @@ class LBaaSDriver(DriverBase):
         raise NotImplementedError()
 
     @abc.abstractmethod
-    def release_listener(self, endpoints, loadbalancer, listener):
+    def release_listener(self, loadbalancer, listener):
         """Release listener.
 
         Should return without errors if listener or load balancer does not
         exist (e.g. already deleted).
 
-        :param endpoints: dict containing K8s Endpoints object
         :param loadbalancer: `LBaaSLoadBalancer` object
         :param listener: `LBaaSListener` object
         """
         raise NotImplementedError()
 
     @abc.abstractmethod
-    def ensure_pool(self, endpoints, loadbalancer, listener):
-        """Get or create pool.
+    def ensure_pool(self, loadbalancer, listener):
+        """Get or create pool attached to Listener.
 
-        :param endpoints: dict containing K8s Endpoints object
         :param loadbalancer: `LBaaSLoadBalancer` object
         :param listener: `LBaaSListener` object
         """
         raise NotImplementedError()
 
     @abc.abstractmethod
-    def release_pool(self, endpoints, loadbalancer, pool):
+    def ensure_pool_attached_to_lb(self, loadbalancer, namespace,
+                                   svc_name, protocol):
+        """Get or create pool attached to LoadBalancer.
+
+        :param loadbalancer: `LBaaSLoadBalancer` object
+        :param namespace: K8S service's namespace
+        :param svc_name: K8S service's name
+        :param protocol: pool's protocol
+        """
+        raise NotImplementedError()
+
+    @abc.abstractmethod
+    def get_loadbalancer_pool_name(self, loadbalancer, namespace, svc_name):
+        """Get name of a load balancer's pool attached to LB.
+
+        The pool's name should be unique per K8S service
+
+        :param loadbalancer: `LBaaSLoadBalancer` object
+        :param namespace: K8S service's namespace
+        :param svc_name: K8S service's name
+        """
+        raise NotImplementedError()
+
+    @abc.abstractmethod
+    def release_pool(self, loadbalancer, pool):
         """Release pool.
 
         Should return without errors if pool or load balancer does not exist
         (e.g. already deleted).
 
-        :param endpoints: dict containing K8s Endpoints object
         :param loadbalancer: `LBaaSLoadBalancer` object
         :param pool: `LBaaSPool` object
         """
         raise NotImplementedError()
 
     @abc.abstractmethod
-    def ensure_member(self, endpoints, loadbalancer, pool,
-                      subnet_id, ip, port, target_ref):
+    def ensure_member(self, loadbalancer, pool,
+                      subnet_id, ip, port, target_ref_namespace,
+                      target_ref_name):
         """Get or create member.
 
-        :param endpoints: dict containing K8s Endpoints object
         :param loadbalancer: `LBaaSLoadBalancer` object
         :param pool: `LBaaSPool` object
         :param subnet_id: Neutron subnet ID of the target
         :param ip: target's IP (e.g. Pod's IP)
         :param port: target port
-        :param target_ref: Kubernetes ObjectReference of the target (e.g.
-                           Pod reference)
+        :param target_ref_namespace: Kubernetes EP target_ref namespace
+        :param target_ref_name: Kubernetes EP target_ref name
         """
         raise NotImplementedError()
 
     @abc.abstractmethod
-    def release_member(self, endpoints, loadbalancer, member):
+    def release_member(self, loadbalancer, member):
         """Release member.
 
         Should return without errors if memberor load balancer does not exist
         (e.g. already deleted).
 
-        :param endpoints: dict containing K8s Endpoints object
         :param loadbalancer: `LBaaSLoadBalancer` object
         :param member: `LBaaSMember` object
         """
         raise NotImplementedError()
 
+    @abc.abstractmethod
+    def get_lb_by_uuid(self, lb_uuid):
+        """Get loadbalancer by loadbalancer uuid.
+
+        :param lb_uuid: Loadbalancer uuid
+        """
+        raise NotImplementedError()
+
+    @abc.abstractmethod
+    def get_pool_by_name(self, pool_name, project_id):
+        """Get pool by pool's name.
+
+        :param pool_name: the pool name
+        :param project_id: project id
+        """
+        raise NotImplementedError()
+
+    @abc.abstractmethod
+    def ensure_l7_policy(self, namespace, route_name, loadbalancer,
+                         pool, listener_id):
+        """Get or create L7 policy.
+
+        :param namespace: ocp-route/k8s-ingress namespace
+        :param route_name: ocp-route/k8s-ingress name
+        :param loadbalancer: `LBaaSLoadBalancer` object
+        :param pool: L7 policy's target pool
+        :param listener_id: ID of listener to attach L7policy to
+
+        """
+        raise NotImplementedError()
+
+    @abc.abstractmethod
+    def release_l7_policy(self, loadbalancer, l7_policy):
+        """Release l7policy.
+
+        :param loadbalancer: `LBaaSLoadBalancer` object
+        :param l7_policy: `LBaaSL7Policy` object
+
+        """
+        raise NotImplementedError()
+
+    @abc.abstractmethod
+    def ensure_l7_rule(self, loadbalancer, l7_policy, compare_type,
+                       type, value):
+        """Get or create L7 rule.
+
+        :param loadbalancer: `LBaaSLoadBalancer` object
+        :param l7_policy: `LBaaSL7Policy` object
+        :param compare_type: comparison type for the L7 rule.
+        :param type: the L7 rule type
+        :param value:the value to use for the comparison.
+
+        """
+        raise NotImplementedError()
+
+    @abc.abstractmethod
+    def release_l7_rule(self, loadbalancer, l7_rule):
+        """Release L7 rule.
+
+        :param loadbalancer: `LBaaSLoadBalancer` object
+        :param l7_rule: `LBaaSL7Rule` object
+
+        """
+        raise NotImplementedError()
+
+    @abc.abstractmethod
+    def update_l7_rule(self, l7_rule, new_value):
+        """Update L7 rule value.
+
+        :param l7_rule: `LBaaSL7Rule` object
+        :param new_value: rule's new value
+
+        """
+        raise NotImplementedError()
+
+    @abc.abstractmethod
+    def is_pool_used_by_other_l7policies(l7policy, pool):
+        """Checks if pool used by other L7policy.
+
+        :param l7policy: `LBaaSL7Policy` object
+        :param pool: `LBaaSPool` object
+
+        """
+        raise NotImplementedError()
+
 
 @six.add_metaclass(abc.ABCMeta)
 class VIFPoolDriver(PodVIFDriver):
@@ -467,3 +688,44 @@ class ServicePubIpDriver(DriverBase):
         :param service_pub_ip_info: service loadbalancer IP info
 
         """
+
+
+@six.add_metaclass(abc.ABCMeta)
+class NetworkPolicyDriver(DriverBase):
+    """Provide network-policy for pods"""
+
+    ALIAS = 'network_policy'
+
+    @abc.abstractmethod
+    def ensure_network_policy(self, policy, project_id):
+        """Policy created or updated
+
+        :param policy: dict containing Kubernetes NP object
+        :param project_id: openstack project_id
+        """
+        raise NotImplementedError()
+
+    @abc.abstractmethod
+    def release_network_policy(self, policy, project_id):
+        """Delete a network policy
+
+        :param policy: dict containing Kubernetes NP object
+        :param project_id
+        """
+        raise NotImplementedError()
+
+
+@six.add_metaclass(abc.ABCMeta)
+class NetworkPolicyProjectDriver(DriverBase):
+    """Get an OpenStack project id for K8s network policies"""
+
+    ALIAS = 'network_policy_project'
+
+    @abc.abstractmethod
+    def get_project(self, policy):
+        """Get an OpenStack project id for K8s pod ports.
+
+        :param policy: dict containing Kubernetes NP object
+        :returns: OpenStack project_id
+        """
+        raise NotImplementedError()
